#ifndef __TaylorsData_Calendar_h__
#define __TaylorsData_Calendar_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <Base4c/Date.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export Calendar : public Base
	{
		public:
			class TaylorsData_Export OpenStatus : public Base
			{
				public:
					class TaylorsData_Export Key
					{
						private:
							int _nOpenStatus;

						public:
							Key();
							Key( int nOpenStatus );
							Key( const Key& key );
							virtual ~Key();

							Key& operator=( const Key& key );
							bool operator==( const Key& key ) const;
							bool operator!=( const Key& key ) const;
							bool operator<( const Key& key ) const;

							int openStatus() const;

							std::string text() const;
					};

				private:
					Key _key;
					std::string _strDescription;
					std::string _strDecoration;

				public:
					OpenStatus();
					OpenStatus( const Key& key );
					OpenStatus( const OpenStatus& status );
					virtual ~OpenStatus();

					OpenStatus& operator=( const OpenStatus& status );

					const Key& key() const;
					const std::string& description() const;
					const std::string& decoration() const;

					void setDescription( const std::string& strDescription );
					void setDecoration( const std::string& strDecoration );

					static OpenStatus fromRow(const RfcDal::DalDatabase::Row& row);
					static OpenStatus create( const OpenStatus& type, RfcDal::DalDatabase& conn );
					virtual void update( RfcDal::DalDatabase& conn );
			};

			class TaylorsData_Export OpenStatusMap : public std::map<OpenStatus::Key, OpenStatus>
			{
				public:
					static OpenStatusMap fromResult( const RfcDal::DalDatabase::Result& result );
			};

			class TaylorsData_Export OpenDay : public Base
			{
				public:
					class TaylorsData_Export Key
					{
						private:
							int _nOpenDay;

						public:
							Key();
							Key( int nOpenDay );
							Key( const Key& key );
							virtual ~Key();

							Key& operator=( const Key& key );
							bool operator==( const Key& key ) const;
							bool operator!=( const Key& key ) const;
							bool operator<( const Key& key ) const;

							int openDay() const;

							std::string text() const;
					};

				private:
					Key _key;
					const OpenStatus* _pstatus;

				public:
					OpenDay();
					OpenDay( const Key& key );
					OpenDay( const OpenDay& day );
					virtual ~OpenDay();

					OpenDay& operator=( const OpenDay& day );

					const Key& key() const;
					const OpenStatus& openStatus() const; // throws if pointer NULL

					void setOpenStatus( const OpenStatus& status );

					static OpenDay fromRow(const RfcDal::DalDatabase::Row& row);
					static OpenDay create( const OpenDay& type, RfcDal::DalDatabase& conn );
					virtual void update( RfcDal::DalDatabase& conn );
			};

			class TaylorsData_Export OpenDayMap : public std::map<OpenDay::Key, OpenDay>
			{
				public:
					static OpenDayMap fromResult( const RfcDal::DalDatabase::Result& result );
			};

			class TaylorsData_Export OpenDayException : public Base
			{
				public:
					class TaylorsData_Export Key
					{
						private:
							Base4c::Date _dtException;

						public:
							Key();
							Key( const Base4c::Date& dtException );
							Key( const Key& key );
							virtual ~Key();

							Key& operator=( const Key& key );
							bool operator==( const Key& key ) const;
							bool operator!=( const Key& key ) const;
							bool operator<( const Key& key ) const;

							const Base4c::Date& openDayException() const;

							std::string text() const;
					};

				private:
					Key _key;
					const OpenStatus* _pstatus;
					std::string _strLegend;

				public:
					OpenDayException();
					OpenDayException( const Key& key );
					OpenDayException( const OpenDayException& day );
					virtual ~OpenDayException();

					OpenDayException& operator=( const OpenDayException& day );

					const Key& key() const;
					const OpenStatus& openStatus() const;
					const std::string& legend() const;

					void setOpenStatus( const OpenStatus& status );
					void setLegend( const std::string& strLegend );

					static OpenDayException fromRow(const RfcDal::DalDatabase::Row& row);
					static OpenDayException create( const OpenDayException& type, RfcDal::DalDatabase& conn );
					virtual void update( RfcDal::DalDatabase& conn );
					virtual void remove( RfcDal::DalDatabase& conn );
			};

			class TaylorsData_Export OpenDayExceptionMap : public std::map<OpenDayException::Key, OpenDayException>
			{
				public:
					static OpenDayExceptionMap fromResult( const RfcDal::DalDatabase::Result& result );
			};

		private:
			static OpenStatusMap _openStatusS;
			static OpenDayMap _openDayS;

		public:
			static void initialize( RfcDal::DalDatabase& conn );
			static OpenStatusMap& openStatuss();
			static OpenDayMap& openDays();

			static OpenDayExceptionMap calendar( int nYear, int nMonth );
	};
}

#endif /* !__TaylorsData_Calendar_h__ */
