#ifndef __TaylorsData_Format_h__
#define __TaylorsData_Format_h__

#include <TaylorsData/TaylorsData.h>

#include <TaylorsData/Base.h>

#include <librfcdal/DalDatabase.h>

#include <string>
#include <map>

namespace TaylorsData
{
	class TaylorsData_Export Format : public Base
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strFormat;

				public:
					Key();
					Key( const std::string& strFormat );
					Key( const Key& key );
					virtual ~Key();

					const Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& format() const;
			};

			static const Key keyDefault;

		private:
			Key _key;
			std::string _strDescription;
			std::string _strFormatTemplate;
			std::string _strTargetDirectory;
			std::string _strTargetFile;
			std::string _strTargetExtension;

		public:
			Format();
			Format( const Key& key );
			Format( const Format& format );
			virtual ~Format();

			const Format& operator=( const Format& format );

			const Key& key() const;
			const std::string& description() const;
			const std::string& formatTemplate() const;
			const std::string& targetDirectory() const;
			const std::string& targetFile() const;
			const std::string& targetExtension() const;
			std::string target() const;

			void setDescription( const std::string& strDescription );
			void setFormatTemplate( const std::string& strFormatTemplate );
			void setTargetDirectory( const std::string& strTargetDirectory );
			void setTargetFile( const std::string& strTargetFile );
			void setTargetExtension( const std::string& strTargetExtension );

			static Format fromRow( const RfcDal::DalDatabase::Row& row, RfcDal::DalDatabase& db );
			static Format create( const Format& format, RfcDal::DalDatabase& conn );
			virtual void update( RfcDal::DalDatabase& conn );
	};

	class TaylorsData_Export FormatMap : public std::map<Format::Key, Format>
	{
		public:
			static FormatMap fromResult( const RfcDal::DalDatabase::Result& result, RfcDal::DalDatabase& db );
	};
};

#endif /* !__TaylorsData_Format_h__ */
