#ifndef __TaylorsData_Settings_h__
#define __TaylorsData_Settings_h__

#include <TaylorsData/TaylorsData.h>
#include <TaylorsData/Exception.h>

#include <Base4c/Version.h>

#include <librfc/XMLNode.hpp>

#include <string>
#include <map>
#include <set>
#include <fstream>

class QMainWindow;

namespace TaylorsData
{
	class TaylorsData_Export Tool
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strTool;

				public:
					Key();
					Key( const std::string& strTool );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& tool() const;
			};

		private:
			Key _key;
			QMainWindow* (*_fnTool)();
			const std::string& (*_fnLegend)();
			const Base4c::Version& (*_fnVersion)();

		public:
			Tool();
			Tool( const Key& key, QMainWindow* (*fnTool)(), const std::string& (*fnLegend)(), const Base4c::Version& (*fnVersion)() );
			Tool( const Tool& tool );
			virtual ~Tool();

			Tool& operator=( const Tool& tool );

			const Key& key() const;
			QMainWindow* tool() const;
			const std::string& legend() const;
			const Base4c::Version& version() const;
	};

	class ToolMap : public std::map<Tool::Key, Tool>
	{
	};

//	class TaylorsData_Export ToolMap : public std::map<std::string, QMainWindow*(*)()>
//	{
//	};

	class TaylorsData_Export ToolSet : public std::set<std::string>
	{
	};

	class TaylorsData_Export Setting
	{
		public:
			class TaylorsData_Export Key
			{
				private:
					std::string _strComponent;
					std::string _strName;

				public:
					Key();
					Key( const std::string& strComponent, const std::string& strName );
					Key( const Key& key );
					virtual ~Key();

					Key& operator=( const Key& key );
					bool operator==( const Key& key ) const;
					bool operator!=( const Key& key ) const;
					bool operator<( const Key& key ) const;

					const std::string& component() const;
					const std::string& name() const;

					std::string asText() const;
					static Key fromText( const std::string& strText );
			};

			enum Type{ TypeNew, TypeFreeText, TypeDate, TypeDirectoryPath,
					TypePathName, TypeURI, TypeReport };

//			static const int TypeNew;
//			static const int TypeFreeText;
//			static const int TypeDate;
//			static const int TypeFilePath;
//			static const int TypePathName;
//			static const int TypeURI;
//			static const int TypeReport;

		private:
			Key _key;
			std::string _strLegend;
			Type _type;
			std::string _strAdditionalInfo;
			std::string _strValue;

		public:
			Setting();
			Setting( const Key& key );
			Setting( const Setting& setting );
			virtual ~Setting();

			Setting& operator=( Setting& setting );

			const Key& key() const;
			const std::string& legend() const;
			Type type() const;
			const std::string& additionalInfo() const;
			const std::string& value() const;

			void setLegend( const std::string& strLegend );
			void setType( Type type );
			void setAdditionalInfo( const std::string& strInfo );
			void setValue( const std::string& strValue );

			static Type typeFromInt( int nType );

			void toXML( Rfc::XMLNode* pnodeParent ) const;
			static Setting fromXML( const Rfc::XMLNode* pnode );
	};

	class TaylorsData_Export SettingMap : public std::map<Setting::Key, Setting>
	{
	};

	class TaylorsData_Export Settings
	{
		private:
			static std::string _strFile;
			static std::string _strDefaultReport;
			static ToolSet _objectS;
			static ToolMap _toolS;
			static SettingMap _settingS;

		public:
			static void load( const std::string& strFile );
			static void loadAscii( std::ifstream& h, char* psz, size_t nC );
			static void loadXml( std::ifstream& h, char* psz, size_t nC );
			static void loadTool( const std::string& strSO );

//			static const std::string& defaultReport(); // To be deprecated
			static SettingMap& settings();
			static Setting& setting( const Setting::Key& key );
			static const std::string& value( const Setting::Key& key );

			static const ToolSet& objects();
			static const ToolMap& tools();

//			static void setDefaultReport( const std::string& strReport ); // To be deprecated

			static void addObject( const std::string& strSO );
			static void removeObject( const std::string& strSO );

			static void save();
	};
}

#endif // __TaylorsData_Settings_h__
